function expe_param_noise
%EXPE_PARAM_NOISE Experiments on the parameters that account for the noise
%with the Richard's method.

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

sz_grid = 100;
n_corresp = 75;

% Noise amplitude (expressed in pixels in the input image)
noise_amp = 1;
% Noise distribution
noise_type = 'gaussian';

% General experiment parameters
EXPE_SURF_MIN = 9;
EXPE_SURF_MAX = 100;
EXPE_CORRESP_MIN = 1;
EXPE_CORRESP_MAX = 1;

%all_perc = [0 5 10 20 40 60 80 100 150 200];
%all_perc = [0 5 10 20 40 60 80 100 150 200];
all_perc = [0:10:60 80 100 140 180];

for i_expe_surf = EXPE_SURF_MIN:EXPE_SURF_MAX
    rand('twister', i_expe_surf); %#ok<RAND>
    randn('state', i_expe_surf); %#ok<RAND>

    % Generate a random piece of paper (without auto-occultations)
    [grid_2d_tpl grid_2d_img grid_3d cam_mat] = ris_rand_paper(sz_grid, true);

    for i_expe_corresp = EXPE_CORRESP_MIN:EXPE_CORRESP_MAX
        rand('twister', 100000*i_expe_surf+i_expe_corresp); %#ok<RAND>
        randn('state', 100000*i_expe_surf+i_expe_corresp); %#ok<RAND>
    
        % Extract some point corespondences
        [corresp_2d_tpl corresp_2d_img corresp_3d] = ...
            ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d);

        % Add noise
        [corresp_2d_tpl corresp_2d_img] = ...
            ris_add_noise(corresp_2d_tpl, corresp_2d_img, noise_type, noise_amp);

        all_err_r1 = zeros(numel(all_perc));
        all_err_r2 = zeros(numel(all_perc));
        
        for i_noise_tpl = 1:numel(all_perc)
            for i_noise_img = 1:numel(all_perc)
                % Noise parameters for the reconstruction algorithms
                noise_tpl = all_perc(i_noise_tpl) * noise_amp / 100;
                noise_img = all_perc(i_noise_img) * noise_amp / 100;

                ok_r1 = true;
                ok_r2 = true;

                % Richard's idea, in term of pixels
                try
                    pts_3d_r1 = ris_richard_errimg( ...
                        corresp_2d_tpl, corresp_2d_img, ...
                        cam_mat, noise_tpl, noise_img);
                catch excep
                    if strcmp(excep.identifier, 'ris_richard_errimg:infeasible')
                        ok_r1 = false;
                    end
                end

                % Richard's idea, in term of vectors
                try
                    pts_3d_r2 = ris_richard_errvect( ...
                        corresp_2d_tpl, corresp_2d_img, ...
                        cam_mat, noise_tpl, noise_img/3600);
                catch excep
                    if strcmp(excep.identifier, 'ris_richard_errvect:infeasible')
                        ok_r2 = false;
                    end
                end


                if ok_r1
                    err_r1 = mean(sqrt(sum((corresp_3d - pts_3d_r1).^2)));
                else
                    err_r1 = -1;
                end
                all_err_r1(i_noise_tpl, i_noise_img) = err_r1;

                if ok_r2
                    err_r2 = mean(sqrt(sum((corresp_3d - pts_3d_r2).^2)));
                else
                    err_r2 = -1;
                end
                all_err_r2(i_noise_tpl, i_noise_img) = err_r2;
                
                fprintf('(%d,%d) r1=%f  r2=%f\n', i_noise_tpl, i_noise_img, err_r1, err_r2);
            end
        end
        path_out = sprintf('expe_param_noise/surf%d-corresp%d.mat', i_expe_surf, i_expe_corresp);
        
        save(path_out, 'all_err_r1', 'all_err_r2', 'sz_grid', 'n_corresp', 'noise_amp', 'noise_type', 'all_perc');
    end
end



% figure(1);
% plot(grid_2d_tpl(1,:), grid_2d_tpl(2,:), '.');
% daspect([1 1 1])
% hold on;
% plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), 'r.', 'MarkerSize', 16);
% 
% figure(2);
% plot(grid_2d_img(1,:), grid_2d_img(2,:), '.');
% daspect([1 1 1])
% hold on;
% plot(corresp_2d_img(1,:), corresp_2d_img(2,:), 'r.', 'MarkerSize', 16);
