function expe_param_noise_salzmann
%EXPE_PARAM_NOISE_SALZMANN Test of the Salzmann's method with the same data
%as the ones used in "expe_param_noise". However, the purpose of this
%experiment is not the same: here, I'm just interested by the
%reconstruction error, not by the influence of the "noise parameters" of
%the Richard's methods.

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

sz_grid = 100;
%n_corresp = 75;
%n_corresp = 150;
n_corresp = 225;

% Noise amplitude (expressed in pixels in the input image)
noise_amp = 1;
% Noise distribution
noise_type = 'gaussian';

% General experiment parameters
EXPE_SURF_MIN = 1;
EXPE_SURF_MAX = 1000;
EXPE_CORRESP_MIN = 1;
EXPE_CORRESP_MAX = 1;

all_lbd = [0.001 0.01 0.1:0.2:1 1.5:0.5:15 16:20];
%all_lbd = [0.01 0.1 1];

for i_expe_surf = EXPE_SURF_MIN:EXPE_SURF_MAX
    rand('twister', i_expe_surf); %#ok<RAND>
    randn('state', i_expe_surf); %#ok<RAND>

    % Generate a random piece of paper (without auto-occultations)
    [grid_2d_tpl grid_2d_img grid_3d cam_mat] = ris_rand_paper(sz_grid, true);

    for i_expe_corresp = EXPE_CORRESP_MIN:EXPE_CORRESP_MAX
        rand('twister', 100000*i_expe_surf+i_expe_corresp); %#ok<RAND>
        randn('state', 100000*i_expe_surf+i_expe_corresp); %#ok<RAND>
    
        % Extract some point corespondences
        [corresp_2d_tpl corresp_2d_img corresp_3d] = ...
           ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d);

        % Add noise
        [corresp_2d_tpl corresp_2d_img] = ...
           ris_add_noise(corresp_2d_tpl, corresp_2d_img, noise_type, noise_amp);
        
        % Initialization for the Salzmann's method
        umin = min(min(grid_2d_tpl(1,:)), min(corresp_2d_tpl(1,:))) - 1;
        umax = max(max(grid_2d_tpl(1,:)), max(corresp_2d_tpl(1,:))) + 1;
        vmin = min(min(grid_2d_tpl(2,:)), min(corresp_2d_tpl(2,:))) - 1;
        vmax = max(max(grid_2d_tpl(2,:)), max(corresp_2d_tpl(2,:))) + 1;
        npts_mesh = 10;
        [tri tri_x tri_y tri_z] = ris_create_tri_mesh(umin, umax, npts_mesh, vmin, vmax, npts_mesh);

        % Convert the template points into barycentric coordinates
        [b1 b2 b3 ind_tri] = ris_cart_to_bary( ...
            tri, tri_x, tri_y, tri_z, ... 
            corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), ones(size(corresp_2d_tpl(2,:))));

        % Build the inextensibility pairs
        thres_dist = 1.25 * sqrt(2*(max(umax-umin, vmax-vmin) / (npts_mesh - 1))^2);
        [inext_pairs dist_pairs] = ris_make_inext_pairs(tri_x, tri_y, thres_dist);

        all_err = zeros(1, numel(all_lbd));
        for i_lbd = 1:numel(all_lbd)
            lbd = all_lbd(i_lbd);
            
            % Reconstruction
            % (x,y,z) are the Cartesian coordinates of the mesh _vertices_
            [~, x, y, z] = ris_salzmann_err( ...
                                    tri, npts_mesh^2, ...
                                    ind_tri, b1, b2, b3, ...
                                    corresp_2d_img, ...
                                    cam_mat, ...
                                    inext_pairs, dist_pairs, ...
                                    ones(1,n_corresp), lbd);

            [xx yy zz] = bary_to_cart(tri, x, y, z, ind_tri, b1, b2, b3);
            pts_3d = [xx ; yy ; zz];
        
            err = mean(sqrt(sum((corresp_3d - pts_3d).^2)));
            all_err(i_lbd) = err;
            
            fprintf('(surf=%d,lbd=%f) err=%f\n', i_expe_surf, lbd, err);
        end
        
        path_out = sprintf('expe_param_noise_salzmann_225pts/surf%d-corresp%d.mat', i_expe_surf, i_expe_corresp);
        
        save(path_out, 'all_err', 'all_lbd', 'sz_grid', 'n_corresp', 'noise_amp', 'noise_type');
    end
end


function [x y z] = bary_to_cart(tri, tx, ty, tz, ind_tri, b1, b2, b3)
% Evaluate the 3D position of the data points
for i = 1:numel(b1)
    i1 = tri(ind_tri(i), 1);
    i2 = tri(ind_tri(i), 2);
    i3 = tri(ind_tri(i), 3);
    x(i) = b1(i) * tx(i1) + b2(i) * tx(i2) + b3(i) * tx(i3); %#ok<AGROW>
    y(i) = b1(i) * ty(i1) + b2(i) * ty(i2) + b3(i) * ty(i3); %#ok<AGROW>
    z(i) = b1(i) * tz(i1) + b2(i) * tz(i2) + b3(i) * tz(i3); %#ok<AGROW>
end
