function [corresp_2d_tpl corresp_2d_img] = ris_add_noise(corresp_2d_tpl, corresp_2d_img, type, ampl)
%RIS_ADD_NOISE Add noise to 2D point correspondences.
%
% SYNTAX
%  [corresp_2d_tpl corresp_2d_img] = ...
%   ris_add_noise(corresp_2d_tpl, corresp_2d_img, type, ampl)
%
% DESCRIPTION
%  Add noise to 2D point correspondences with an exact control of the 
%  amplitude. The correspondences are typically generated with
%  "ris_extract_corresp". The same type of noise (with the same amplitude)
%  is added to both the correspondences in the template and in the input 
%  image. It is specifically designed for the experiments on the RIS
%  toolbox. In particular, noise amlitudes are adjusted to fit with the
%  typical dimensions of the images.
%
% INPUT ARGUMENTS
%  - corresp_2d_tpl, corresp_2d_img [2 x n_corresp matrices]: point
%     correspondences.
%  - type [string]:
%     * 'gaussian'
%     * 'uniform'
%  - ampl [scalar]: amplitude of the noise, expressed in pixels in the
%     input image.
%
% OUTPUT ARGUMENT
%  - corresp_2d_tpl, corresp_2d_img [2 x n_corresp matrices]: point
%     correspondences with noise added.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

n_corresp = size(corresp_2d_tpl, 2);

theta_tpl = 2*pi * rand(1, n_corresp);
theta_img = 2*pi * rand(1, n_corresp);

ampl_tpl = ampl/4;
ampl_img = ampl;

switch type
    case 'gaussian'
        fact_tpl = ampl_tpl * randn(1, n_corresp);
        fact_img = ampl_img * randn(1, n_corresp);
        
    case 'uniform'
        fact_tpl = ampl_tpl * rand(1, n_corresp) - ampl_mm/2;
        fact_img = ampl_img * rand(1, n_corresp) - ampl_mm/2;
        
    otherwise
        error('ris_add_noise:noise_type', 'Unknown name for the noise distribution');
end

corresp_2d_tpl(1,:) = corresp_2d_tpl(1,:) + fact_tpl .* cos(theta_tpl);
corresp_2d_tpl(2,:) = corresp_2d_tpl(2,:) + fact_tpl .* sin(theta_tpl);

corresp_2d_img(1,:) = corresp_2d_img(1,:) + fact_img .* cos(theta_img);
corresp_2d_img(2,:) = corresp_2d_img(2,:) + fact_img .* sin(theta_img);
