function [val jacob JtJ] = ris_cost_jacob( ...
    corresp_2d_tpl, corresp_2d_img, lbd_err3d, ...
    pts_inext_img, ind_pts_inext, dist_inext, lbd_inext, ...
    mat_intr, bbs, ...
    sq_bendmat, Omega_ij, ...
    parameters)
%RIS_COST_JACOB Cost function to be given to "lsqnonlin" for reconstructing
%inextensible 3D surfaces from monocular vision.
%
% This function is similar to "ris_cost" except for these differences:
%  - there is one line for each of the term (the norms are not merged, the
%    bending term is "spread", etc.);
%  - for the inextensibility terms, we compare the squared distances
%    instead of the distances themselves;
%  - the (exact) jacobian matrix can be computed.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

%% Pre-computation
n_corresp = size(corresp_2d_img, 2);

% Extraction of the diffrent parts of the parameters
[depth ctrlpts ctrlpts_x ctrlpts_y ctrlpts_z] = ...
    ris_split_parameters(parameters, n_corresp);

n_ctrlpts = numel(ctrlpts_x);
n_inext = numel(dist_inext);

%% 3D error

% Homogeneous coordinates of the 2D points in the image
corresp_2d_img_hom = [corresp_2d_img ; ones(1, n_corresp)];

% Inverse of the intrinsic matrix of the camera
cam_intr_inv = inv(mat_intr);

% 3D points from the warp
corresp_3d = bbs_eval(bbs, ctrlpts, corresp_2d_tpl(1,:)', corresp_2d_tpl(2,:)');


cost_error_3d = lbd_err3d * reshape(corresp_3d ...
    - repmat(depth, 3, 1) .* (cam_intr_inv * corresp_2d_img_hom), ...
    3*n_corresp, 1); %#ok<MINV>


%% Inextensibility constraints

% 3D warp of the inextensibility points
pts_inext_3d = bbs_eval(bbs, ctrlpts, pts_inext_img(1,:), pts_inext_img(2,:));

% Current (squared) distance between the 3D inextensibility points
dist_inext_3d = sum((pts_inext_3d(:,ind_pts_inext(1,:)) ...
                     - pts_inext_3d(:,ind_pts_inext(2,:))).^2);

cost_inext = sqrt(lbd_inext) * (dist_inext_3d - dist_inext.^2)';


%% Bending energy term
cost_bend = [sq_bendmat * ctrlpts_x' ; ... 
             sq_bendmat * ctrlpts_y' ; ...
             sq_bendmat * ctrlpts_z'];
         

%% Final cost function
val = [cost_error_3d ; cost_inext ; cost_bend];


%% Jacobian matrix
if nargout >= 2
    %% 3D error
    
    % Collocation matrix of the point correspondences
    coloc = bbs_coloc(bbs, corresp_2d_tpl(1,:)', corresp_2d_tpl(2,:)');
             
    jac_err_3d_i = zeros(1, 17*n_corresp);
    jac_err_3d_j = zeros(1, 17*n_corresp);
    jac_err_3d_s = zeros(1, 17*n_corresp);
    deb = 1;
    for i = 1:n_corresp
        jac_err_3d_i(deb:deb+2) = 3*(i-1)+(1:3);
        jac_err_3d_j(deb:deb+2) = i;
        jac_err_3d_s(deb:deb+2) = cam_intr_inv*corresp_2d_img_hom(:,i); %#ok<MINV>
        deb = deb + 3;
        
        w = coloc(i,:);
        
        [tmp_i tmp_j tmp_s] = find(w);
        n_nz = 3*numel(tmp_s);
        jac_err_3d_i(deb:deb+n_nz-1) = [3*(i-1)+tmp_i 3*(i-1)+tmp_i+1 3*(i-1)+tmp_i+2];
        jac_err_3d_j(deb:deb+n_nz-1) = [n_corresp+tmp_j n_corresp+n_ctrlpts+tmp_j n_corresp+2*n_ctrlpts+tmp_j];
        jac_err_3d_s(deb:deb+n_nz-1) = [tmp_s tmp_s tmp_s];
        
        deb = deb + n_nz;
    end
    jac_err_3d = lbd_err3d * sparse(jac_err_3d_i, jac_err_3d_j, jac_err_3d_s, ...
                          3*n_corresp, n_corresp+3*n_ctrlpts);


    %% Inextensibility constraints
    ctrlpts_ln = sqrt(lbd_inext) * [ctrlpts_x ctrlpts_y ctrlpts_z];
    jac_inext_i = zeros(1,3*32*n_inext);
    jac_inext_j = zeros(1,3*32*n_inext);
    jac_inext_s = zeros(1,3*32*n_inext);
    deb = 1;
    for k = 1:n_inext
        Om = Omega_ij{k};
        tmp = ctrlpts_ln * Om;
        
        [~, tmp_j, tmp_s] = find(tmp);
        n_nz = numel(tmp_s);

        jac_inext_i(deb:deb+n_nz-1) = k*ones(1,n_nz);
        jac_inext_j(deb:deb+n_nz-1) = n_corresp+tmp_j;
        jac_inext_s(deb:deb+n_nz-1) = tmp_s;
        
        deb = deb + n_nz;
    end
    jac_inext = sparse(jac_inext_i(1:deb-1), jac_inext_j(1:deb-1), jac_inext_s(1:deb-1), ...
                       n_inext, n_corresp+3*n_ctrlpts);

    

    %% Bending energy
    Zd = sparse(n_ctrlpts, n_corresp);
    Z = sparse(n_ctrlpts, n_ctrlpts);
    B = sparse(sq_bendmat);
    jac_bend = [Zd B Z Z ; Zd Z B Z ; Zd Z Z B];
    
    jacob = [jac_err_3d ; jac_inext ; jac_bend];
    
    
    %% (Clever?) computation of jacob' * jacob
    if nargout >= 3
        JtJ = jac_err_3d'*jac_err_3d + jac_inext'*jac_inext + jac_bend'*jac_bend;
    end
end

