function mat_bary = ris_mat_bary(tri, nvert, ind_tri, b1, b2, b3)
%RIS_MAT_BARY Creates a sparse matrix
%
% SYNTAX
%  mat_bary = ris_mat_bary(tri, nvert, ind_tri, b1, b2, b3)
%
% DESCRIPTION
%  Creates the matrix B_i as defined after Eq. (2) in "M. Salzmann, P. Fua.
%  Reconstructing Sharply Folding Surfaces: A Convex Formulation. CVPR, 
%  2009." (sect 3.1, p.3, column 1).
%
% INPUT ARGUMENTS
%  - tri [nx3 matrix]: matrix that defines the triangular mesh (as returned 
%      by "ris_create_tri_mesh", for example).
%  - nvert [scalar]: number of vertices in the triangular mesh.
%  - ind_tri, b1, b2, b3: barycentric coordinates as returned
%      by "ris_bary_to_cart".
%
% OUTPUT ARGUMENTS
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

% Number of facets
ntri = size(tri, 1);

% Number of points
npts = numel(b1);

sp_i = zeros(1, 9*npts);
sp_j = zeros(1, 9*npts);
sp_v = zeros(1, 9*npts);

for k = 1:npts
    ind = 9*(k-1)+1:9*(k-1)+9; % indices in the sp_X variables
    
    ind_vert = tri(ind_tri(k), :);  % indices of the vertex of the current face
    tmp_j1 = 3*(ind_vert(1)-1)+1:3*(ind_vert(1)-1)+3;
    tmp_j2 = 3*(ind_vert(2)-1)+1:3*(ind_vert(2)-1)+3;
    tmp_j3 = 3*(ind_vert(3)-1)+1:3*(ind_vert(3)-1)+3;
    sp_j(ind) = [tmp_j1 tmp_j2 tmp_j3];
    
    sp_i(ind) = repmat(3*(k-1)+1:3*(k-1)+3, 1, 3);
    
    sp_v(ind) = vect(repmat([b1(k) b2(k) b3(k)], 3, 1))';
end

mat_bary = sparse(sp_i, sp_j, sp_v, 3*npts, 3*nvert);
