function [grid_2d_tpl grid_2d_img, grid_3d, cam_mat] = ris_rand_paper(sz_grid, no_occl)
%RIS_RAND_PAPER Generate a random piece of paper, specifically designed for
%the experiments on the RIS toolbox.
%
% SYNTAX
%  [grid_2d_tpl grid_2d_img, grid_3d] = ris_rand_paper(sz_grid)
%  [grid_2d_tpl grid_2d_img, grid_3d, cam_mat] = ris_rand_paper(sz_grid)
%  [...] = ris_rand_paper(sz_grid, no_occl)
%
% DESCRIPTION
%  Generates a random piece of paper.
%  The results are grid of points of size "sz_grid" x "sz_grid". Random
%  correspondences ca easily be extracted from this representation using
%  the function "ris_extract_corresp".
%  There is no much choice for parameterizing the results. Default values
%  are given to generate data for the experiments on the RIS toolbox.
%  Heavily relies on the "paper" toolbox of M. Perriollat.
%
% INPUT ARGUMENTS
%  - sz_grid [scalar]: size of the grid (sz_grid x sz_grid)
%  - no_occult [boolean, optional, default=false]: indicates whether
%     auto-occlusions are allowed or not. If true, it may be long to
%     compute (it actually generates random surfaces until we don't have
%     any auto-occlusion).
%
% OUTPUT ARGUMENTS
%  - grid_2d_tpl, grid_2d_img [2 x sz_grid matrix]
%  - grid_3d [3 x sz_grid matrix]
%  - cam_mat [3 x 4 matrix, optional]  
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

% Input arguments
if nargin == 1
    no_occl = false;
end

% set the path
path_old = path;
path_paper = 'D:/PHD/PUBLICATIONS/ACCV2010/socp3d/packagesMP/paper';
path([path_paper '/model'], path);
path([path_paper '/misc'], path);
path([path_paper '/other'], path);

% Parameters of the camera
% We consider that the center of the camera is the origin of the world (and
% thus, there is no translation nor rotation)
K = [100*36 0 0 ; 0 100*36 0 ; 0 0 1];

if nargout >= 4
    cam_mat = [K zeros(3,1)];
end

if ~no_occl
    % Compute the paper
    paper = randPaper(1);

    [u v] = meshgrid(linspace(-1/8,1/8,sz_grid));
    u = u(:)';
    v = v(:)';
    base_3d = uv2M([u ; v], paper);
else
    paper_found = false;
    thres = 0.01;
    n_iter = 1;
    
    while ~paper_found
        paper = randPaper(1);

        [u v] = meshgrid(linspace(-1/8,1/8,sz_grid));
        u = u(:)';
        v = v(:)';
        base_3d = uv2M([u ; v], paper);
        x = reshape(base_3d(1,:), sz_grid, sz_grid);
        y = reshape(base_3d(2,:), sz_grid, sz_grid);
        z = reshape(base_3d(3,:), sz_grid, sz_grid);
        
        ok = true;
        for i = 1:sz_grid-1
            for j = 1:sz_grid-1
                v1 = 800 * [x(i,j) ; y(i,j) ; z(i,j)];
                v2 = 800 * [x(i,j+1) ; y(i,j+1) ; z(i,j+1)];
                v3 = 800 * [x(i+1,j+1) ; y(i+1,j+1) ; z(i+1,j+1)];
                v1(3) = v1(3) + 1000;
                v2(3) = v2(3) + 1000;
                v3(3) = v3(3) + 1000;
                
                u1 = v2 - v1;
                u2 = v3 - v1;
                
                n = normc([ ...
                    u1(2)*u2(3) - u1(3)*u2(2) ;
                    u1(3)*u2(1) - u1(1)*u2(3) ;
                    u1(1)*u2(2) - u1(2)*u2(1)]);
                
                b = normc((v1 + v2 + v3) / 3);
                
                scal = n' * b;
                
                if scal < thres
                    ok = false;
                    break;
                end
                
                v1 = 800 * [x(i,j) ; y(i,j) ; z(i,j)];
                v2 = 800 * [x(i+1,j+1) ; y(i+1,j+1) ; z(i+1,j+1)];
                v3 = 800 * [x(i+1,j) ; y(i+1,j) ; z(i+1,j)];
                v1(3) = v1(3) + 1000;
                v2(3) = v2(3) + 1000;
                v3(3) = v3(3) + 1000;
                
                u1 = v2 - v1;
                u2 = v3 - v1;
                
                n = normc([ ...
                    u1(2)*u2(3) - u1(3)*u2(2) ;
                    u1(3)*u2(1) - u1(1)*u2(3) ;
                    u1(1)*u2(2) - u1(2)*u2(1)]);
                
                b = normc((v1 + v2 + v3) / 3);
                
                scal = n' * b;
                
                if scal < thres
                    ok = false;
                    break;
                end
            end
            if ~ok
                break;
            end
        end
        
        if ok
            paper_found = true;
        else
            n_iter = n_iter + 1;
        end
    end
end

grid_2d_tpl = 800 * [u ; v];

grid_3d = 800 * base_3d;
grid_3d(3,:) = grid_3d(3,:) + 1000;

tmp = K * grid_3d;
grid_2d_img = tmp(1:2,:) ./ repmat(tmp(3,:), 2, 1);

% Restore the path
path(path_old);
