function [depth x y z] = ris_salzmann( ...
    tri, n_vert, ... Triangular mesh
    corresp_ind, corresp_b1, corresp_b2, corresp_b3, ... Barycentric coordinates of the point correspondences
    corresp_2d_img, ... Cartesian coordinates of the pts corresp. in the image
    cam_mat_img, ... Camera matrix
    inext_pairs, dist_pairs) % Pairs for the inextensibility constraints
%RIS_SALZMANN Inextensible 3D surface reconstruction.
%
% SYNTAX
%  [depth x y z] = ris_salzmann( ...
%                     tri, ...
%                     corresp_ind, corresp_b1, corresp_b2, corresp_b3, ...
%                     corresp_2d_img, ...
%                     cam_mat_img, ...
%                     inext_pairs, dist_pairs)
%
% DESCRIPTION
%  Implementation of the noise-free method for 3D inextensible surface
%  reconstruction (part 3.1 of "M. Salzmann, P. Fua. Reconstructing Sharply
%  Folding Surfaces: A Convex Formulation. CVPR, 2009.")
%
% INPUT ARGUMENTS
%  - tri [3 x n_facets matrix]: Triangular mesh
%  - n_vert [scalar]: Number of vertices in the triangular mesh
%  - corresp_ind, corresp_b1, corresp_b2, corresp_b3: Barycentric 
%     coordinates of the point correspondences
%  - corresp_2d_img [2 x n_corresp]: Cartesian coordinates of the pts 
%     correspondences in the image
%  - cam_mat_img [3x4 matrix]: Camera matrix
%  - inext_pairs [2 x n_inext], dist_pairs [1 x n_inext]: Pairs for the 
%     inextensibility constraints
%
% OUTPUT ARGUMENTS
%  - depth [n_corresp x 1]: Depths of the point correspondences.
%  - x, y, z: Cartesan coordinates of the optimized mesh.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

%% Pre-computations
n_corresp = size(corresp_2d_img, 2); % number of point correspondences
n_inext = numel(dist_pairs);  % number of inextensibility constraints


%% Camera and sightlines
iM = inv(cam_mat_img(:,1:3));
%cam_center = -iM * cam_mat_img(:,4); % the camera center
sightlines = normc(iM * [corresp_2d_img ; ones(1,n_corresp)]); %#ok<MINV> % the sightline directions

% %% Modelization of the problem
% cvx_begin
%     variables X(3*n_vert) d(n_corresp)
%     
%     maximize( ones(1,n_corresp) * d )
%     
%     subject to
%         % 3D points must lie on the sight lines
%         for i = 1:n_corresp
%             % Barycentric coordinates of the current point
%             b1 = corresp_b1(i);
%             b2 = corresp_b2(i);
%             b3 = corresp_b3(i);
%             
%             % Indices of the 3 vertices of the face to which the current
%             % point belongs
%             ind = tri(corresp_ind(i), :);
%             i1 = 3*(ind(1)-1)+1;
%             i2 = 3*(ind(2)-1)+1;
%             i3 = 3*(ind(3)-1)+1;
%             
%             b1*X(i1:i1+2) + b2*X(i2:i2+2) + b3*X(i3:i3+2) == d(i)*sightlines(:,i); %#ok<EQEFF>
%             d(i) > 0; %#ok<VUNUS>
%         end
%         
%         % Inextensibility constraints
%         for k = 1:n_inext
%             i = 3*(inext_pairs(1,k)-1)+1;
%             j = 3*(inext_pairs(2,k)-1)+1;
%             norm(X(i:i+2)-X(j:j+2), 2) <= dist_pairs(k); %#ok<VUNUS>
%         end
% cvx_end
% 
% depth = d;
% [x y z] = ris_vect_to_coord(X);

f = [sparse(3*n_vert,1) ; -ones(n_corresp,1)];

% The 3D points must lie on the sight lines
for k = 1:n_corresp
    B_i = ris_mat_bary(tri, n_vert, corresp_ind(k), corresp_b1(k), corresp_b2(k), corresp_b3(k));
    tmp = sparse(3,n_corresp);
    tmp(:,k) = -sightlines(:,k);
    
    A{k} = [B_i tmp];
    c{k} = sparse(3,1);
    b{k} = sparse(3*n_vert+n_corresp,1);
    d{k} = 2;
end

% Inextensibility constraints
k = n_corresp + 1;
for i = 1:n_inext
    ii = 3*(inext_pairs(1,i)-1)+1;
    jj = 3*(inext_pairs(2,i)-1)+1;

    ind_i = [1  2    3    1  2    3];
    ind_j = [ii ii+1 ii+2 jj jj+1 jj+2];
    ind_v = [1  1    1    -1 -1   -1];
    
    A{k} = sparse(ind_i, ind_j, ind_v, 3, 3*n_vert+n_corresp);
    c{k} = sparse(3,1);
    b{k} = sparse(3*n_vert+n_corresp,1);
    d{k} = dist_pairs(i);
    
    k = k + 1;
end

[At, bt, ct, K] = ris_readable_to_sedumi(f, A, c, b, d);

[~, ys] = sedumi(At, bt, ct, K);
[x y z] = ris_vect_to_coord(ys(1:3*n_vert));
depth = ys(3*n_vert+1:end);
