function [depth x y z] = ris_salzmann_iter( ...
    tri, n_vert, ... Triangular mesh
    corresp_ind, corresp_b1, corresp_b2, corresp_b3, ... Barycentric coordinates of the point correspondences
    corresp_2d_img, ... Cartesian coordinates of the pts corresp. in the image
    cam_mat_img, ... Camera matrix
    inext_pairs, dist_pairs) % Pairs for the inextensibility constraints
%RIS_SALZMANN_ITER Inextensible 3D surface reconstruction with noise in the 
%image.
%
% SYNTAX
%  [depth x y z] = ris_salzmann( ...
%                     tri, ...
%                     corresp_ind, corresp_b1, corresp_b2, corresp_b3, ...
%                     corresp_2d_img, ...
%                     cam_mat_img, ...
%                     inext_pairs, dist_pairs)
%
% DESCRIPTION
%  Implementation of the method for 3D inextensible surface reconstruction
%  with noise in the image (part 3.1 of "M. Salzmann, P. Fua. 
%  Reconstructing Sharply Folding Surfaces: A Convex Formulation. CVPR, 
%  2009.")
%  This is the iterative process.
%
% INPUT ARGUMENTS
%  - tri [3 x n_facets matrix]: Triangular mesh
%  - n_vert [scalar]: Number of vertices in the triangular mesh
%  - corresp_ind, corresp_b1, corresp_b2, corresp_b3: Barycentric 
%     coordinates of the point correspondences
%  - corresp_2d_img [2 x n_corresp]: Cartesian coordinates of the pts 
%     correspondences in the image
%  - cam_mat_img [3x4 matrix]: Camera matrix
%  - inext_pairs [2 x n_inext], dist_pairs [1 x n_inext]: Pairs for the 
%     inextensibility constraints
%
% OUTPUT ARGUMENTS
%  - depth [n_corresp x 1]: Depths of the point correspondences.
%  - x, y, z: Cartesan coordinates of the optimized mesh.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

n_corresp = numel(corresp_b1);
weights = ones(1, n_corresp);
n_iter = 1;

while true
    [depth, x, y, z] = ris_salzmann_err( ...
        tri, n_vert, ... 
        corresp_ind, corresp_b1, corresp_b2, corresp_b3, ...
        corresp_2d_img, ...
        cam_mat_img, ...
        inext_pairs, dist_pairs, ...
        weights);

    [xx yy zz] = bary_to_cart(tri, x, y, z, corresp_ind, corresp_b1, corresp_b2, corresp_b3);

    tmp = cam_mat_img(:,1:3) * [xx ; yy ; zz];
    reproj_2d_img = tmp(1:2,:) ./ repmat(tmp(3,:), 2, 1);

    err_reproj = sqrt(sum((corresp_2d_img - reproj_2d_img).^2));
    old_weights = weights;
    weights = exp(-err_reproj./mean(err_reproj));
    
    fprintf('|w|=%f   repro=%f   depth=%f\n', norm(weights), mean(err_reproj), mean(depth));
    
    if norm(old_weights-weights) <= 1.0e-4 || n_iter >= 50
        break;
    end

    n_iter = n_iter + 1;
end



function [x y z] = bary_to_cart(tri, tx, ty, tz, ind_tri, b1, b2, b3)
% Evaluate the 3D position of the data points
for i = 1:numel(b1)
    i1 = tri(ind_tri(i), 1);
    i2 = tri(ind_tri(i), 2);
    i3 = tri(ind_tri(i), 3);
    x(i) = b1(i) * tx(i1) + b2(i) * tx(i2) + b3(i) * tx(i3); %#ok<AGROW>
    y(i) = b1(i) * ty(i1) + b2(i) * ty(i2) + b3(i) * ty(i3); %#ok<AGROW>
    z(i) = b1(i) * tz(i1) + b2(i) * tz(i2) + b3(i) * tz(i3); %#ok<AGROW>
end
