function [depths, bounds3D] = ris_socp_cvx(pointsTemplate, pointsImage, camera, noiseLevelMax)
%RIS_SOCP_CVX Inextensible 3D surface reconstruction using SOCP and CVX.
%
% Compute the 3D reconstruction of an inextensible surface, giving point
% correspondances between the current image of the deformed surface and 
% the flat template. Uses an SOCP formulation of the problem.
%
% Input arguments
%
%  - pointsTemplate [2xn matrix]: matrix giving the points in the template.
%
%  - pointsImage [2xn matrix]: matrix giving the points in the current
%     image.
%
%  - camera [3x4 matrix]: matrix giving the projective camera matrix.
%
%  - noiseLevelMax [scalar, optional (default=0)]: maximal noise level.
%
% Output arguments
%
%  - depths [1xn matrix]: matrix giving the depth of the points along their 
%     sightline.
%
%  - bounds3D [3xn matrix]: matrix giving the 3D bound points.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

%% Initial computations

% Maximum level of noise
if nargin < 4
    noiseLevelMax = 0;
end

% Number of points correspondences
nPoints = size(pointsTemplate, 2);

% Pairewise distances between the points in the template. 
% "distTemplate(i,j)" is the euclidean distance between the points
% "pointsTemplate(:,i)" and "pointsTemplate(:,j)".
distTemplate = squareform(pdist(pointsTemplate')) + noiseLevelMax;

% Initial computation on the camera
iM = inv(camera(:, 1:3));
camCenter = -iM * camera(:, 4); % the camera center
sightLines = normc(iM * [pointsImage ; ones(1, nPoints)]); %#ok<MINV> % the sightline directions
% (A point Qi along the sightline i is: Qi = \mu*v_i + C)
fid = fopen('tmp.txt', 'w');
cvx_begin
    variable x(nPoints)
    maximize( sum(x) )
    subject to
    
cvx_end
fclose(fid);
depths = M;

bounds3D = repmat(camCenter, 1, nPoints) + repmat(depths', 3, 1) .* sightLines;



%         for i = 1:nPoints
%             for j = i+1:nPoints
%                 if distTemplate(i,j) <= 32
%                     %A = [sightLines(:,i) -sightLines(:,j)];
%                     A = sparse(3, nPoints);
%                     A(:,i) = sightLines(:,i);
%                     A(:,j) = -sightLines(:,j);
%                     %norm(A * M, 2) <= distTemplate(i,j)+noiseLevelMax;
%                     %norm(M(i)*sightLines(:,i) - M(j)*sightLines(:,j), 2) <= distTemplate(i,j)+noiseLevelMax; %#ok<VUNUS>
%                     fprintf(fid, 'norm([');
%                     fprintf(fid, '%f ', full(A(1,:)));
%                     fprintf(fid, ' ; ');
%                     fprintf(fid, '%f ', full(A(2,:)));
%                     fprintf(fid, ' ; ');
%                     fprintf(fid, '%f ', full(A(3,:)));
%                     fprintf(fid, ']*M ) <= %f\n', distTemplate(i,j)+noiseLevelMax);
%                 end
%             end
%             %M(i) >= 0;
%         end